//structure file for TIFFs, used by okteta hex editor (part of KDE)
//     Copyright (C) 2016/2017 Andreas Romeyke
//
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//


function tiffEndianessValues() {
  // name="endianess_t" type="UInt16">
  //  <entry name="MagicByte (big endian)" value="0x4D4D" byteOrder="bigEndian" />
  //  <entry name="MagicByte (little endian)" value="0x4949" byteOrder="littleEndian"/>
    if (this.value == 0x4d4d) parent.byteorder(big-endian);
    else if (this.value == 0x4949) parent.byteorder(little-endian);
    return {
    "MagicByte (big endian)" : 0x4D4D,
    "MagicByte (little endian)" : 0x4949
    };
}

function tiffType() {
  return {
    "BYTE" : 1,
    "ASCII" : 2,
    "SHORT" : 3,
    "LONG" : 4,
    "RATIONAL" : 5,
    "SBYTE" : 6,
    "UNDEFINED" : 7,
    "SSHORT" : 8,
    "SLONG" : 9,
    "SRATIONAL" : 10,
    "FLOAT" : 11,
    "DOUBLE" : 12,
  };
}

// tags converted by author from checkit_tiff, 
//       see file checkit_tiff/src/helper/check_tiffparse.c
//       the checkit_tiff will be found at 
//       https://github.com/SLUB-digitalpreservation/checkit_tiff
function tiffTags() {
return {
    "SubFileType" : 254,
    "OldSubFileType" : 255,
    "ImageWidth" : 256,
    "ImageLength" : 257,
    "BitsPerSample" : 258,
    "Compression" : 259,
    "Photometric" : 262,
    "Threshholding" : 263,
    "CellWidth" : 264,
    "CellLength" : 265,
    "FillOrder" : 266,
    "DocumentName" : 269,
    "ImageDescription" : 270,
    "Make" : 271,
    "Model" : 272,
    "StripOffsets" : 273,
    "Orientation" : 274,
    "SamplesPerPixel" : 277,
    "RowsPerStrip" : 278,
    "StripByteCounts" : 279,
    "MinSampleValue" : 280,
    "MaxSampleValue" : 281,
    "XResolution" : 282,
    "YResolution" : 283,
    "PlanarConfig" : 284,
    "PageName" : 285,
    "XPosition" : 286,
    "YPosition" : 287,
    "FreeOffsets" : 288,
    "FreeByteCounts" : 289,
    "GrayResponseUnit" : 290,
    "GrayResponseCurve" : 291,
    "Group3Options" : 292,
    "Group4Options" : 293,
    "ResolutionUnit" : 296,
    "PageNumber" : 297,
    "ColorResponseUnit" : 300,
    "TransferFunction" : 301,
    "Software" : 305,
    "DateTime" : 306,
    "Artist" : 315,
    "HostComputer" : 316,
    "Predictor" : 317,
    "Whitepoint" : 318,
    "PrimaryChromaticities" : 319,
    "Colormap" : 320,
    "HalftoneHints" : 321,
    "TileWidth" : 322,
    "TileLength" : 323,
    "TileOffsets" : 324,
    "TileByteCounts" : 325,
    "BadFaxLines" : 326,
    "CleanFaxData" : 327,
    "ConsecutiveBadFaxLines" : 328,
    "SubIFD" : 330,
    "InkSet" : 332,
    "InkNames" : 333,
    "NumberOfInks" : 334,
    "DotRange" : 336,
    "TargetPrinter" : 337,
    "ExtraSamples" : 338,
    "SampleFormat" : 339,
    "SMinSampleValue" : 340,
    "SMaxSampleValue" : 341,
    "JPEGTables" : 347,
    "GLOBALPARAMETERSIFD" : 400,
    "PROFILETYPE" : 401,
    "FAXPROFILE" : 402,
    "CODINGMETHODS" : 403,
    "VERSIONYEAR" : 404,
    "MODENUMBER" : 405,
    "DECODE" : 433,
    "IMAGEBASECOLOR" : 434,
    "T82OPTIONS" : 435,
    "JPEGProcessingMode" : 512,
    "JPEGInterchangeFormat" : 513,
    "JPEGInterchangeFormatLength" : 514,
    "JPEGRestartInterval" : 515,
    "JPEGLosslessPredictors" : 517,
    "JPEGPointTransform" : 518,
    "JPEGQTables" : 519,
    "JPEGDCTables" : 520,
    "JPEGACTables" : 521,
    "YCbCrCoefficients" : 529,
    "YCbCrSubsampling" : 530,
    "YCbCrPositioning" : 531,
    "ReferenceBlackWhite" : 532,
    "StripRowCounts" : 559,
    "XMP" : 700,
    "inofficial EXIF ThumbnailData (poss. Olympus)" : 20507,
    "inofficial EXIF ThumbnailCompression (poss. Olympus)" : 20515,
    "inofficial EXIF ThumbnailResolutionX (poss. Olympus)" : 20525,
    "inofficial EXIF ThumbnailResolutionY (poss. Olympus)" : 20526,
    "inofficial EXIF ThumbnailResolutionUnit (poss. Olympus)" : 20528,
    "inofficial EXIF LuminanceTable (poss. Olympus)" : 20624,
    "inofficial EXIF ChrominanceTable (poss. Olympus)" : 20625,
    "OLD BOGUS Matteing tag" : 32768,
    "OpiImageId" : 32781,
    "XIS private compression scheme tag" : 32835,
    "XIS private compression scheme tag" : 32836,
    "XIS private compression scheme tag" : 32837,
    "XIS private compression scheme tag" : 32838,
    "XIS private compression scheme tag" : 32839,
    "TIFF HiTIFF / Wang TIFF" : 32931,
    "TIFF annotation data / Wang TIFF" : 32932,
    "unknown Wang TIFF" : 32933,
    "Offset to Page Control Structure / Wang TIFF" : 32934,
    "TOC / Wang TIFF" : 32935,
    "unknown Ig (Island Graphics)" : 32952,
    "IgReferencePoints (Island Graphics)" : 32953,
    "IgRegionTackPoint (Island Graphics)" : 32954,
    "IgRegionWarpCorners (Island Graphics)" : 32955,
    "IgRegionAffine (Island Graphics)" : 32956,
    "temperature (COMPIX)" : 32970,
    "Zoombox (COMPIX)" : 32971,
    "WinTES (COMPIX)" : 32972,
    "OBSOLETE Matteing (Silicon Graphics)" : 32995,
    "OBSOLETE DataType (Silicon Graphics)" : 32996,
    "ImageDepth (Silicon Graphics)" : 32997,
    "TileDepth (Silicon Graphics)" : 32998,
    "PIXAR_IMAGEFULLWIDTH" : 33300,
    "PIXAR_IMAGEFULLLENGTH" : 33301,
    "PIXAR_TEXTUREFORMAT" : 33302,
    "PIXAR_WRAPMODES" : 33303,
    "PIXAR_FOVCOT" : 33304,
    "PIXAR_MATRIX_WORLDTOSCREEN" : 33305,
    "PIXAR_MATRIX_WORLDTOCAMERA" : 33306,
    "Kodak serial number" : 33405,
    "CFAREPEATPATTERNDIM" : 33421,
    "CFAPATTERN" : 33422,
    "BatteryLevel (TIFF/EP)" : 33423,
    "Kodak IFD" : 33424,
    "Copyright" : 33432,
    "EXIF ExposureTime" : 33434,
    "EXIF FNumber" : 33437,
    "MD_Filetag" : 33445,
    "MD_ScalePixel" : 33446,
    "MD_Colortable" : 33447,
    "MD_LabName" : 33448,
    "MD_SampleInfo" : 33449,
    "MD_PrepDate" : 33450,
    "MD_PrepTime" : 33451,
    "MD_FileUnits" : 33452,
    "SIS Init Standard Metadata (Olympus)" : 33471,
    "GeoTIFF ModelPixelScale" : 33550,
    "SIS Offset Metadata (Olympus)" : 33560,
    "Component subsample CMYK/YCBCR (WangTIFF)" : 33609,
    "MetaMorph Stack Image UIC1" : 33628,
    "MetaMorph Stack Image UIC2" : 33629,
    "MetaMorph Stack Image UIC3" : 33630,
    "MetaMorph Stack Image UIC4" : 33631,
    "RichTIFFIPTC / NAA" : 33723,
    "INGR Packet Data Tag  (Intergraph)" : 33918,
    "INGR Flag Registers  (Intergraph)" : 33919,
    "IrasB Transformation Matrix / obsolete GeoTIFF IntergraphMatrixTag (Intergraph)" : 33920,
    "Georeferencing Tag / obsolete (Intergraph)" : 33921,
    "GeoTIFF ModelTiepointTag (Intergraph)" : 33922,
    "Leica Magic Tag" : 33923,
    "XIS private tag" : 33979,
    "XIS private tag" : 33980,
    "XIS private tag" : 33981,
    "XIS private tag" : 33982,
    "XIS private tag" : 33983,
    "IT8SITE" : 34016,
    "IT8COLORSEQUENCE" : 34017,
    "IT8HEADER" : 34018,
    "IT8RASTERPADDING" : 34019,
    "IT8BITSPERRUNLENGTH" : 34020,
    "IT8BITSPEREXTENDEDRUNLENGTH" : 34021,
    "IT8COLORTABLE" : 34022,
    "IT8IMAGECOLORINDICATOR" : 34023,
    "IT8BKGCOLORINDICATOR" : 34024,
    "IT8IMAGECOLORVALUE" : 34025,
    "IT8BKGCOLORVALUE" : 34026,
    "IT8PIXELINTENSITYRANGE" : 34027,
    "IT8TRANSPARENCYINDICATOR" : 34028,
    "IT8COLORCHARACTERIZATION" : 34029,
    "IT8HCUSAGE" : 34030,
    "IT8TRAPINDICATOR" : 34031,
    "IT8CMYKEQUIVALENT" : 34032,
    "RichTIFFIPTC / NAA (deprecated)" : 34152,
    "FRAMECOUNT" : 34232,
    "Geotiff ModelTransformationTag (JPL Carto Group)" : 34264,
    "Leaf MOS PKTS tag (Raw camera)" : 34310,
    "MMHeader (Olympus Fluoview or Andor Bio-imaging Division)" : 34361,
    "MMStamp (Olympus Fluoview or Andor Bio-imaging Division)" : 34362,
    "Photoshop Image Ressources" : 34377,
    "EXIFIFDOffset" : 34665,
    "ICC Profile" : 34675,
    "Annotation Offsets (Wang TIFF)" : 34730,
    "Mask SUBIFDs (Wang TIFF)" : 34731,
    "ImageLayer" : 34732,
    "Geotiff GeoKeyDirectoryTag (SPOT)" : 34735,
    "Geotiff GeoDoubleParamsTag (SPOT)" : 34736,
    "Geotiff GeoAsciiParamsTag (SPOT)" : 34737,
    "JBIG Options" : 34750,
    "EMC/PixTool SpecificTag" : 34820,
    "EXIF ExposureProgram" : 34850,
    "EXIF SpectralSensitivity" : 34852,
    "GPSIFD or Olympus SIS TAG2" : 34853,
    "EXIF ISOSpeedRatings" : 34855,
    "EXIF OECF" : 34856,
    "Interlace (TIFF/EP)" : 34857,
    "TimeZoneOffset (TIFF/EP)" : 34858,
    "SelfTimerMode (TIFF/EP)" : 34859,
    "HylaFax FaxRecvParams" : 34908,
    "HylaFax FaxSubAdress" : 34909,
    "HylaFax FaxRecvTime" : 34910,
    "EXIF ExifVersion" : 36864,
    "EXIF DateTimeOriginal" : 36867,
    "EXIF DateTimeDigitized" : 36868,
    "EXIF ComponentsConfiguration" : 37121,
    "EXIF CompressedBitsPerPixel" : 37122,
    "EXIF ShutterSpeedValue" : 37377,
    "EXIF ApertureValue" : 37378,
    "EXIF BrightnessValue" : 37379,
    "EXIF ExposureBiasValue" : 37380,
    "EXIF MaxApertureValue" : 37381,
    "EXIF SubjectDistance" : 37382,
    "EXIF MeteringMode" : 37383,
    "EXIF LightSource" : 37384,
    "EXIF Flash" : 37385,
    "EXIF FocalLength" : 37386,
    "FlashEnergy (TIFF/EP)" : 37387,
    "SpatialFrequencyResponse (TIFF/EP)" : 37388,
    "Noise (TIFF/EP)" : 37389,
    "FocalPlaneXResolution (TIFF/EP)" : 37390,
    "FocalPlaneYResolution (TIFF/EP)" : 37391,
    "FocalPlaneResolutionUnit (TIFF/EP)" : 37392,
    "ImageNumber (TIFF/EP)" : 37393,
    "SecurityClassification (TIFF/EP)" : 37394,
    "ImageHistory (TIFF/EP)" : 37395,
    "EXIF SubjectArea" : 37396,
    "ExposureIndex (TIFF/EP)" : 37397,
    "TIFF/EP StandardID" : 37398,
    "Sensing Method (TIFF/EP)" : 37399,
    "old Kodak KDC" : 37400,
    "StoNits" : 37439,
    "EXIF MakerNote" : 37500,
    "EXIF UserComment" : 37510,
    "EXIF SubsecTime" : 37520,
    "EXIF SubsecTimeOriginal" : 37521,
    "EXIF SubsecTimeDigitized" : 37522,
    "Plain Text (Microsoft Office Document Image Writer)" : 37679,
    "OLE Property Set Storage (Microsoft Office Document Image Writer)" : 37680,
    "Plain Text Positioning Information (Microsoft Office Document Image Writer)" : 37681,
    "ImageSourceData" : 37724,
    "SatelliteName (NinjoSat)" : 40001,
    "DateID (NinjoSat)" : 40002,
    "CreationDateID (NinjoSat)" : 40003,
    "ChannelID (NinjoSat)" : 40004,
    "HeaderVersion (NinjoSat)" : 40005,
    "FileName (NinjoSat)" : 40006,
    "DataType (NinjoSat)" : 40007,
    "SatelliteNumber (NinjoSat)" : 40008,
    "ColorDepth (NinjoSat)" : 40009,
    "XMinimum (NinjoSat)" : 40011,
    "XMaximum (NinjoSat)" : 40012,
    "YMinimum (NinjoSat)" : 40013,
    "YMaximum (NinjoSat)" : 40014,
    "code (NinjoSat)" : 40015,
    "PhysicValue (NinjoSat)" : 40024,
    "PhysicUnit (NinjoSat)" : 40025,
    "MinGrayValue (NinjoSat)" : 40026,
    "MaxGrayValue (NinjoSat)" : 40027,
    "Gradient (NinjoSat)" : 40028,
    "AxisIntercept (NinjoSat)" : 40029,
    "XPTitle (Microsoft)" : 40091,
    "XPComment (Microsoft)" : 40092,
    "XPAuthor (Microsoft)" : 40093,
    "XPKeywords (Microsoft)" : 40094,
    "XPSubject (Microsoft)" : 40095,
    "EXIF FlashpixVersion" : 40960,
    "EXIF ColorSpace" : 40961,
    "EXIF PixelXDimension" : 40962,
    "EXIF PixelYDimension" : 40963,
    "EXIF RelatedSoundFile" : 40964,
    "INTEROPERABILITYIFD" : 40965,
    "EXIF FlashEnergy" : 41483,
    "EXIF SpatialFrequencyResponse" : 41484,
    "EXIF FocalPlaneXResolution" : 41486,
    "EXIF FocalPlaneYResolution" : 41487,
    "EXIF FocalPlaneResolutionUnit" : 41488,
    "EXIF SubjectLocation" : 41492,
    "EXIF ExposureIndex" : 41493,
    "EXIF SensingMethod" : 41495,
    "EXIF FileSource" : 41728,
    "EXIF SceneType" : 41729,
    "EXIF CFAPattern" : 41730,
    "EXIF CustomRendered" : 41985,
    "EXIF ExposureMode" : 41986,
    "EXIF WhiteBalance" : 41987,
    "EXIF DigitalZoomRatio" : 41988,
    "EXIF FocalLengthIn35mmFilm" : 41989,
    "EXIF SceneCaptureType" : 41990,
    "EXIF GainControl" : 41991,
    "EXIF Contrast" : 41992,
    "EXIF Saturation" : 41993,
    "EXIF Sharpness" : 41994,
    "EXIF DeviceSettingDescription" : 41995,
    "EXIF SubjectDistanceRange" : 41996,
    "EXIF ImageUniqueID" : 42016,
    "GDAL Metadata" : 42112,
    "GDAL nodata" : 42113,
    "Imacon 3F" : 46275,
    "Imacon 3F raw dim" : 46279,
    "Oce Scanjob Description" : 50215,
    "Oce Application Selector" : 50216,
    "Oce Identification Number" : 50217,
    "Oce ImageLogic Characteristics" : 50218,
    "Photoshop Annotations" : 50255,
    "inofficial EXIF PrintImageMatching (poss. Olympus)" : 50341,
    "unknown Sinar tag" : 50454,
    "unknown Sinar tag" : 50455,
    "Hasselblad IFD" : 50459,
    "DNGVERSION" : 50706,
    "DNGBACKWARDVERSION" : 50707,
    "UNIQUECAMERAMODEL" : 50708,
    "LOCALIZEDCAMERAMODEL" : 50709,
    "CFAPLANECOLOR" : 50710,
    "CFALAYOUT" : 50711,
    "LINEARIZATIONTABLE" : 50712,
    "BLACKLEVELREPEATDIM" : 50713,
    "BLACKLEVEL" : 50714,
    "BLACKLEVELDELTAH" : 50715,
    "BLACKLEVELDELTAV" : 50716,
    "WHITELEVEL" : 50717,
    "DEFAULTSCALE" : 50718,
    "DEFAULTCROPORIGIN" : 50719,
    "DEFAULTCROPSIZE" : 50720,
    "COLORMATRIX1" : 50721,
    "COLORMATRIX2" : 50722,
    "CAMERACALIBRATION1" : 50723,
    "CAMERACALIBRATION2" : 50724,
    "REDUCTIONMATRIX1" : 50725,
    "REDUCTIONMATRIX2" : 50726,
    "ANALOGBALANCE" : 50727,
    "ASSHOTNEUTRAL" : 50728,
    "ASSHOTWHITEXY" : 50729,
    "BASELINEEXPOSURE" : 50730,
    "BASELINENOISE" : 50731,
    "BASELINESHARPNESS" : 50732,
    "BAYERGREENSPLIT" : 50733,
    "LINEARRESPONSELIMIT" : 50734,
    "CAMERASERIALNUMBER" : 50735,
    "LENSINFO" : 50736,
    "CHROMABLURRADIUS" : 50737,
    "ANTIALIASSTRENGTH" : 50738,
    "SHADOWSCALE" : 50739,
    "DNGPRIVATEDATA" : 50740,
    "MAKERNOTESAFETY" : 50741,
    "CALIBRATIONILLUMINANT2" : 50779,
    "BESTQUALITYSCALE" : 50780,
    "RAWDATAUNIQUEID" : 50781,
    "Alias Layer Metadata" : 50784,
    "ORIGINALRAWFILENAME" : 50827,
    "ORIGINALRAWFILEDATA" : 50828,
    "ACTIVEAREA" : 50829,
    "MASKEDAREAS" : 50830,
    "ASSHOTICCPROFILE" : 50831,
    "ASSHOTPREPROFILEMATRIX" : 50832,
    "CURRENTICCPROFILE" : 50833,
    "CURRENTPREPROFILEMATRIX" : 50834,
    "ImageJ Metadata Bytecounts (Micromanager)" : 50838,
    "ImageJ Metadata (Micromanager)" : 50839,
    "CameraCalibration Sig" : 50931,
    "ProfileCalibration Sig" : 50932,
    "ExtraCameraProfiles IFD" : 50933,
    "AsShotProfileName" : 50934,
    "NoiseReductionApplied" : 50935,
    "ProfileName" : 50936,
    "ProfileHueSatMapDims" : 50937,
    "ProfileHueSatMapData1" : 50938,
    "ProfileHueSatMapData2" : 50939,
    "ProfileToneCurve" : 50940,
    "ProfileEmbedPolicy" : 50941,
    "ProfileEmbedCopyright" : 50942,
    "ForwardMatrix1" : 50964,
    "ForwardMatrix2" : 50965,
    "PreviewApplicationName" : 50966,
    "PreviewApplicationVersion" : 50967,
    "PreviewSettingsName" : 50968,
    "PreviewSettingsDigest" : 50969,
    "PreviewColorSpace" : 50970,
    "PreviewDateTime" : 50971,
    "RawImageDigest" : 50972,
    "OriginalRawFileDigest" : 50973,
    "ProfileLookTableDims" : 50981,
    "ProfileLookTableData" : 50982,
    "NoiseProfile (DNG 1.3)" : 51041,
    "NewRawImageDigest (DNG 1.4)" : 51111,
    "MicroManagerMetadata (Micromanager)" : 51123,
    "Padding/ Exif MS HDPhoto" : 59932,
    "Kodak P-Series" : 64772,

};
}

function tagEntry() {
  var t = struct ( {
    Tag: enumeration("Tag", uint16(), tiffTags()),
    Type: enumeration("Type", uint16(), tiffType()),
    Count: uint32(),
    Value_or_Offset: uint32(),
  });
  t.name = "TIFF Tag";
  return t;
}

function tiffIFD() {
  var ifd =  struct( {
      Count: uint16(),
      Tags: array(tagEntry(), function() { return this.parent.Count.value; }),
      Offset_to_next_IFD: uint32()
    } );
  ifd.name = "TIF IFD";
  return ifd;
} 

function init() {
  var ifdOffset = pointer( uint32(), tiffIFD());
  var tiff_header = struct( {
    Magic_Byte: enumeration("Endianess", uint16(), tiffEndianessValues()),
    Constant_42: enumeration("Constant 42", uint16(), { "42": 42 }),
    Offset_to_IFD0: ifdOffset,
  } );

 
  tiff_header.name="TIFF Header";
  tiff_header.defaultLockOffset = 0x0;
  return tiff_header
}
